import
{
	select,
	csv,
	scaleLinear,
	scaleTime,
	axisLeft,
	axisBottom,
	line,
  max,
  mouse, 
  bisector
}
from 'd3';

const formatLineChartData = data => {
  	var formatted = [];
  	for(var x = 1985; x < 2019; x++) {
        formatted[x] = { "year": new Date(x + ""), "count": 0 };
    }
    data.forEach(d => {
      var country = d["Country of Operator/Owner"];
      var date = new Date(d["Date of Launch"]);
      var year = date.getFullYear();
      formatted[year].count = formatted[year].count + 1;
    });
  
  var results = [];
  for(var x = 0; x < formatted.length; x++) {
    if (formatted[x]) {
      results.push(formatted[x]);
    }
  }
  
  return results;
};

export const lineChart = (selection, props) =>
{
	const
	{
		svg,
		height,
		width,
		yLocation,
    yearSelectCallback
	} = props;

	const xValue = d => d.year;

	const yValue = d => d.count;

	const margin =
	{
		top: 60,
		right: 40,
		bottom: 88,
		left: 105
	};
	const innerWidth = width - margin.left - margin.right;
	const innerHeight = height - margin.top - margin.bottom;

	const xScale = scaleTime()
		.domain([new Date("1985"), new Date("2019")])
		.range([0, innerWidth])
		.nice();
  
  let renderYAxis = (data, g) => {
    let upper = Math.ceil(max(data, d => d.count) / 10) * 10;
    const yScale = scaleLinear()
      .domain([0, upper])
      .range([innerHeight, 0])
      .nice();

		const yAxis = axisLeft(yScale)
			.tickSize(-innerWidth)
			.tickPadding(10);

		const yAxisG = g.call(yAxis);
		yAxisG.selectAll('.domain').remove();
    
    return yScale;
  };
  
  let setupTooltip = g => {
    // The following code is based on the code found here:
    // https://bl.ocks.org/alandunning/cfb7dcd7951826b9eacd54f0647f48d3
    var focus = g.append("g")
        .attr("class", "focus")
        .style("display", "none");
    
    focus.append("line")
        .attr("class", "x-hover-line hover-line")
        .attr("y1", 0)
        .attr("y2", 50);

    focus.append("text")
        .attr("x", 15)
    		.attr('y', 20)
      	.attr("dy", ".31em");

    svg.append("rect")
        .attr("transform", "translate(" + margin.left + "," + (margin.top + yLocation) + ")")
        .attr("class", "overlay")
        .attr("width", width)
        .attr("height", height)
        .on("mouseover", function() { focus.style("display", null); })
        .on("mouseout", function() { focus.style("display", "none"); });
  };
  
  let updateTooltip = (data, xScale, yScale) => {
    function mousemove() {
    	let bisectDate = bisector(function(d) { return d.year; }).left
      var x0 = xScale.invert(mouse(this)[0]),
          i = bisectDate(data, x0, 1),
          d0 = data[i - 1],
          d1 = data[i],
          d = x0 - d0.year > d1.year - x0 ? d1 : d0;
      var focus = svg.select(".focus");
      focus.attr("transform", "translate(" + xScale(d.year) + "," + 0 + ")");
      focus.select("text").text(function() { return d.count; });
    }
    
    function mouseclick() {
    	let bisectDate = bisector(function(d) { return d.year; }).left
      var x0 = xScale.invert(mouse(this)[0]),
          i = bisectDate(data, x0, 1),
          d0 = data[i - 1],
          d1 = data[i],
          d = x0 - d0.year > d1.year - x0 ? d1 : d0;
      let newYearSelected = d.year.getFullYear() + 1;
      
      yearSelectCallback(newYearSelected);
    }
    
    svg.select('.overlay')
        .on("mousemove", mousemove)
    		.on("click", mouseclick)
  }

	let firstRender = data =>
	{
    
		const g = svg.append('g').attr("class", "chart")
			.attr('transform', `translate(${margin.left},${margin.top + yLocation})`);

		const xAxis = axisBottom(xScale)
			.tickSize(-innerHeight)
			.tickPadding(15);

		const xAxisG = g.append('g').call(xAxis)
			.attr('transform', `translate(0,${innerHeight})`);
    
    let yScale = renderYAxis(data, g.append('g').attr("class", "xAxis"));
    setupTooltip(g);
    updateTooltip(data, xScale, yScale);

		xAxisG.select('.domain').remove();

		const lineGenerator = line()
			.x(d => xScale(xValue(d)))
			.y(d => yScale(yValue(d)));

		g.append("g")
		.append("path")
		.datum(data)
		.attr("class", "line")
		.attr("d", lineGenerator);
    
    // Have future render calls only update the line
    renderFn = rerender;
	};
	let rerender = data =>
	{
    let g = svg.select('.chart').select('.xAxis');    
    let yScale = renderYAxis(data, g);
    updateTooltip(data, xScale, yScale);
    
		const lineGenerator = line()
			.x(d => xScale(xValue(d)))
			.y(d => yScale(yValue(d)));

		select(".line")
		.datum(data)
		.attr("d", lineGenerator);

	};
	var renderFn = firstRender;
	return {
		render: data =>
		{
      let formattedData = formatLineChartData(data);
      renderFn(formattedData);
		}
	};
}
